# coding: utf8

"""
plugin.video.freetv addon for Kodi.
This addon provide a video source to acces French TV program from the Freebox.
It use the v3 freebox API (http/json requests) to acces channels and EPG data.
"""

import sys
import re
import urllib
import urlparse
import urllib2
import xbmcgui
import xbmcplugin
import xbmcaddon
import codecs
import os.path
import time
import json

# Names of qualities in requests
QUALITIES = [u"auto",u"hd",u"sd",u"ld"]

# Freebox url
freeboxUrl = 'http://mafreebox.freebox.fr/'

# url of the addon (Kodi format)
urlAddon = sys.argv[0]

# handle of the addon
hAddon = int(sys.argv[1])

# addon class
addon = xbmcaddon.Addon()

# directory of the addon
dLib = xbmc.translatePath( addon.getAddonInfo('path') ).decode("utf-8")

# directory of the addon user data
dProfile = xbmc.translatePath(addon.getAddonInfo('profile')).decode('utf-8')
if not os.path.exists(dProfile):
    os.makedirs(dProfile)

# file name of the bookmarked channels
fBookmarks = dProfile + 'bookmarks.txt'

# Quality user setting
strQuality = addon.getSetting('quality')
if(strQuality is None) or (strQuality == ''):
	quality = QUALITIES[0]
else:
	quality = QUALITIES[int(strQuality)]

# url parameters (mode expected)
argAddon = urlparse.parse_qs(sys.argv[2][1:])

# mode of the addon (None / favorite)
mode = argAddon.get('mode',None)

# list of channels data
channels = []

# list of bookmarked channels (uuid)
bookmarks = []

def localise(id):
	"""
	Utility to use Kodi translation system.
	:param id: int
	"""
	string = addon.getLocalizedString(id)
	return string

def build_url(query):
	"""
	Create an URL for this addon with the specified query
	:param query: str
	:return: str
	"""
	return urlAddon + '?' + urllib.urlencode(query)
	
def get_json(id,url):
	"""
	Create and execute a json request to the freebox.
	Response are cached for 2 hours.
	Return none if the http resquest failed.
	:param url: str
	:return: json
	"""
	
	fCache = dProfile+id+".cache"
	if(os.path.isfile(fCache) and ((time.time()-os.path.getmtime(fCache)) < (60*60*2)) ):
		#Use cache data
		with codecs.open(fCache, "r", "utf-8") as file:
			text = file.read()
			
		return json.loads(text)
	else:
		#Use server data
	
		req = urllib2.Request(url)
		req.add_header('User-Agent', 'Mozilla/5.0 (Windows NT 10.0; WOW64; rv:47.0) Gecko/20100101 Firefox/47.0')
		response = None
	
		try:
			response = urllib2.urlopen(req)
			text=response.read().decode('utf-8')
			response.close()
			
			with codecs.open(fCache, "w", "utf-8") as file:
				file.write(text)
		
			return json.loads(text)
		except:
			if(response is not None):
				response.close()
		
			time.sleep(0.25)
			
		return None
		
def get_epg(timestamp,id):
	"""
	Get the EPG for all channels at the specified timestamp
	:param timestamp: int
	:return: list
	"""
	url = freeboxUrl+'api/v3/tv/epg/by_time/'+timestamp+'/'
	
	jr = get_json(id,url)
	
	if(jr is None):
		dialog = xbmcgui.Dialog()
		dialog.notification('Free TV',localise(30007),xbmcgui.NOTIFICATION_ERROR)
		return None
	elif (not jr['success']):
		dialog = xbmcgui.Dialog()
		dialog.notification('Free TV',localise(30007)+':'+jt['msg'],xbmcgui.NOTIFICATION_ERROR)
		return None
	else:
		return jr['result']

def get_channels_details():
	"""
	Get the details of all channels
	:param timestamp: int
	:return: list
	"""
	url = freeboxUrl+'api/v3/tv/channels/'
	
	jr = get_json('channels',url)
	
	if(jr is None) or (not jr['success']):
		dialog = xbmcgui.Dialog()
		dialog.notification('Free TV',localise(30008),xbmcgui.NOTIFICATION_ERROR)
		return None
	else:
		return jr['result']
		
def channel_number(item):
	"""
	Function used to sort channels by number
	:param item: channel
	:return: int
	"""
	return item['number']
		
def get_channels_streams():
	"""
	Get the streams for all channels
	:param timestamp: int
	:return: list
	"""
	url = freeboxUrl+'api/v3/tv/bouquets/freeboxtv/channels/'
	
	jr = get_json('streams',url)
	
	if(jr is None) or (not jr['success']):
		dialog = xbmcgui.Dialog()
		dialog.notification('Free TV',localise(30009),xbmcgui.NOTIFICATION_ERROR)
		return None
	else:	
		return sorted(jr['result'],key = channel_number)
	
def get_stream_quality(q,streams):
	"""
	Get the best stream for a channel for the desired quality q
	:param q: str
	:param streams: list of stream
	:return: str
	"""
	order = [0,1,2,3]
	if(q == QUALITIES[1]):
		order = [1,2,3,0]
	elif(q == QUALITIES[2]):
		order = [2,3,1,0]
	elif(q == QUALITIES[3]):
		order = [3,2,0,1]
	
	for i in order:
		for stream in streams:
			if(stream['quality'] == QUALITIES[i]):
				return stream['rtsp']

	return streams[0]['rtsp']
		
def epg_date(item):
	"""
	Function used to sort epg event by time
	:param item: channel
	:return: int
	"""
	return item['date']
		
def add_event(lst_event,event):	
	"""
	Add a EPG event in a list of event if not allreadly in it.
	:param lst_event: list of EPG event
	:param event: EPG event
	:return: None
	"""
	for e in lst_event:
		if(e['id'] == event['id']):
			return
	
	lst_event.append(event)
		
def add_channel(channel,lstChannels,epg1,epg2):
	"""
	Add a channel to the channels list.
	
	:param channel: the channel to add
	:param lstChannels: list of the channels details
	:param epg1: list of epg data (this hour)
	:param epg2: list of epg data (hour+1)
	"""
	number = channel['number']
	uuid = channel['uuid']
	label = lstChannels[uuid]['name']
	urlIcon = freeboxUrl+lstChannels[uuid]['logo_url'][1:]
	stream = get_stream_quality(quality,channel['streams'])
	
	info = {'plot':''}
	
	lstEvent = []
	line = ''
	
	if(epg1 is not None):
		if uuid in epg1:
			for e in epg1[uuid]:
				add_event(lstEvent,epg1[uuid][e])
				
	if(epg2 is not None):
		if uuid in epg2:
			for e in epg2[uuid]:
				add_event(lstEvent,epg2[uuid][e])
				
	lstEvent = sorted(lstEvent,key = epg_date)
				
	for p in lstEvent:
		ts = p['date']
		if(ts + p['duration'] > time.time()):	
			date = time.strftime('%H:%M',time.localtime(ts))
			end  = time.strftime('%H:%M',time.localtime(ts+p['duration']))
			line = line + date + '-' + end + ':\n' + p['title'] + '\n\n'
			
	info = {'plot':line}
	
	li = xbmcgui.ListItem(label+' [COLOR pink]('+str(number)+')[/COLOR]', iconImage=urlIcon)
	li.setInfo('video',info)
	
	commands = []
	
	if(uuid in bookmarks):
		commands.append(( localise(30011), 'XBMC.RunScript(%s, remove, %s, %s)' % (os.path.join(dLib, 'resources', 'lib', 'bookmarks.py'), fBookmarks, uuid), ))
	else:
		commands.append(( localise(30010), 'XBMC.RunScript(%s, add, %s, %s)' % (os.path.join(dLib, 'resources', 'lib', 'bookmarks.py'), fBookmarks, uuid), ))
	li.addContextMenuItems(commands, True)
	
	xbmcplugin.addDirectoryItem(handle=hAddon,url=stream,listitem=li)
		
def add_channels(show_all):
	"""
	Add channels to the xbmc list.
	If show_all is True, show all channels.
	Otherwise show only favorites.
	:param show_all: bool
	"""
	now = long(time.time()/3600)*3600
	lt = time.localtime(now)
	hour = lt.tm_hour
	
	lstChannels = get_channels_details()
	if(lstChannels is None):
		return
	
	lstStreams = get_channels_streams()
	if(lstStreams is None):
		return
	
	epg1 = get_epg(str(now),'epg'+str(hour))
	epg2 = get_epg(str(now+3600),'epg'+str(hour+1))
	
	for channel in lstStreams:
		if(channel['available'] and channel['pub_service'] and (channel['sub_number'] == 0)):
			if(show_all):
				add_channel(channel,lstChannels,epg1,epg2)
			elif(channel['uuid'] in bookmarks):
				add_channel(channel,lstChannels,epg1,epg2)

def page_home():
	"""
	Show the home page
	"""
	
	# Link to the favorite page
	url = build_url({'mode' : 'favorite'})
	li = xbmcgui.ListItem(localise(30006))
	xbmcplugin.addDirectoryItem(handle=hAddon,url=url,listitem=li,isFolder=True)

	add_channels(True)
	
	xbmcplugin.endOfDirectory(hAddon)
		
def page_favorite():
	"""
	Show the favorite page
	"""
	add_channels(False)
	
	xbmcplugin.endOfDirectory(hAddon)

"""
Main part of the addon
"""

xbmcplugin.setContent(hAddon,'movies')

if(os.path.isfile(fBookmarks)):
	with codecs.open(fBookmarks, "r", "utf-8") as file:
		lines = file.read()
		bookmarks = lines.splitlines()

if mode is None:
	page_home()
elif mode[0] == 'favorite':
	page_favorite()